var syncMenu = null;

$(function () {

    var inSync = false;
    var nextPosition = null;
    var ignoreNextSync = false;
    var firstMessage = true;

    var menuElement = $("#menu");

    var $tree = $('#tree');
    var scrollElement = $('#content-scroll');
    $tree.tree({
        data: treeData,
        autoEscape: false,
        closedIcon: '&#xe000;',
        openedIcon: '&#xe001;',
        onCreateLi: function (node, $li) {
            if (node.children.length == 0 && node.getLevel() > 1) {
                $li.addClass("tree-icon");
                $li.css({
                    'background': 'url(' + helpTopicImg + ') no-repeat'
                });
            }
        }
    });

    function getTitle(node) {
        return baseTitle + " - " + node.name;
    }

    function navigateToNode(eventNode, createHistory) {
        if (eventNode && eventNode.href && eventNode.digest && eventNode.id && eventNode.name) {
            if (!inSync) {
                var contentElement = document.getElementById('content');
                var targetUrl = relativeBase + eventNode.href;
                if (createHistory && history && history.state) {
                    var previousScrollPos = scrollElement.scrollTop();
                    var previousState = history.state;
                    history.replaceState({href: previousState.href, digest: previousState.digest, name: previousState.name, id: previousState.id, scrollPos: previousScrollPos}, getTitle(previousState));
                }
                $.get(targetUrl + "-" + eventNode.digest + ".cnt", function (data) {
                    contentElement.innerHTML = data;
                    if (eventNode.scrollPos) {
                        scrollElement.scrollTop(eventNode.scrollPos);
                    } else {
                        scrollElement.scrollTop(0);
                    }
                    var title = getTitle(eventNode);
                    setTimeout(function () {
                        document.title = title;
                    }, 50)
                    if (createHistory && history) {
                        history.pushState({href: eventNode.href, digest: eventNode.digest, name: eventNode.name, id: eventNode.id, scrollPos: 0}, title, targetUrl);
                    }
                });
            }
            menuElement.addClass('hidden-small');
        }
    }

    window.onpopstate = function(event) {
        event.preventDefault();
        let state = event.state;
        if (state && state.href && state.digest && state.id && state.name) {
            navigateToNode(state, false);
            syncMenu(state.id)
        }
    };

    $tree.bind('tree.select',
        function (event) {
            navigateToNode(event.node, true);
        }
    );

    $tree.bind('tree.click',
        function(event) {
            if ($tree.tree('isNodeSelected', event.node)) {
                event.preventDefault();
                menuElement.addClass('hidden-small');
            }
            setTreeFocus(true, false);
        }
    );

    var getMenuNode = function (helpId) {
        return $tree.tree('getNodeById', helpId);
    };

    syncMenu = function(helpId, load, position) {
        inSync = !load;
        nextPosition = position;
        try {
            var node = getMenuNode(helpId);
            var currentSelection = $tree.tree('getSelectedNode');
            if (node != currentSelection) {
                $.each($tree.tree('getState').open_nodes, function(index, value) {
                    $tree.tree('closeNode', $tree.tree('getNodeById', value), false)
                });

                $tree.tree('selectNode', node);
                $tree.tree('scrollToNode', node);
            }
            return node;
        } finally {
            inSync = false;
            nextPosition = null;
        }
    }

    $('#menu-button').on("click", function() {
        menuElement.toggleClass('hidden-small');
    });

    var iOS = /iPad|iPhone|iPod/.test(navigator.userAgent) && !window.MSStream;
    if (iOS) {
        $('.ios-scroll').addClass("ios-scroll-impl");
    }

    var currentTreeFocus = false;
    function setTreeFocus(value, changeFocus) {
        $tree.tree('setOption', 'keyboardSupport', value);
        if (changeFocus) {
            if (value) {
                $tree.focus();
            } else {
                scrollElement.focus();
            }
        }
        currentTreeFocus = value;
    }

    $tree.focus(function(event) {
        setTreeFocus(true, false);
    });

    scrollElement.focus(function(event) {
        setTreeFocus(false, false);
    });

    $(document).keydown(function(event) {
        var keycode = event.which;
        switch (keycode) {
            case 115: //F4
                setTreeFocus(!currentTreeFocus, true);
                event.preventDefault();
                break;
            case 74:  //j
                nav("left");
                break;
            case 75:  //k
                nav("right");
                break;
            case 85:  //u
                nav("up");
                break;
            case 68:  //d
                nav("down");
                break;
        }
    });

    function nav(id) {
        $('.nav-' + id).first().each(function() {this.click();})
    }

    $(function () {
        var hash = location.hash;
        if (hash) {
            document.documentElement.scrollTop = document.body.scrollTop = parseInt(hash.substr(1));
        }

        var node = getMenuNode(initialHelpId);
        if (node && history) {
            history.replaceState({href: node.href, digest: node.digest, name: node.name, id: node.id, scrollPos: 0}, document.title, document.location.href);
        }
        syncMenu(initialHelpId, false, null);
        setTreeFocus(false, true)

    });

});

function navigateTo(helpId) {
    syncMenu(helpId, true, null);
}

function showEnlarged(element) {
    var maxWidth = parseInt(element.style.maxWidth, 10);
    if (maxWidth != element.clientWidth) {
        window.location = element.src;
    }
}

function initSvg(element, width, height) {
    if(navigator.userAgent.indexOf('MSIE') !== -1  || navigator.appVersion.indexOf('Trident/') > 0) {
        element.setAttribute("width", width);
        element.setAttribute("height", height);
    }
}

